@@ -45,7 +45,6 @@ enum {
 	tracking_all_motion = 1003,
         tracking_focus = 1004,
         tracking_xterm_ext = 1006,
-        tracking_urxvt = 1015
 };
 
 static int tracking_mode = 0;
@@ -66,7 +65,6 @@ reset_mouse_tracking_mode(void)
 	decset(tracking_cell_motion, FALSE);
 	decset(tracking_all_motion, FALSE);
 	decset(tracking_xterm_ext, FALSE);
-	decset(tracking_urxvt, FALSE);
 	fflush(stdout);
 }
 
@@ -113,10 +111,6 @@ clear(void)
 		fprintf(stdout, "Xterm 1006 mouse tracking extension enabled.\r\n");
 		decset(tracking_xterm_ext, TRUE);
 		break;
-	case tracking_urxvt:
-		fprintf(stdout, "rxvt-unicode 1015 mouse tracking extension enabled.\r\n");
-		decset(tracking_urxvt, TRUE);
-		break;
 	default:
 		fprintf(stdout, "Tracking disabled.\r\n");
 		break;
@@ -128,7 +122,6 @@ clear(void)
 	fprintf(stdout, "D - Cell motion tracking.\r\n");
 	fprintf(stdout, "E - All motion tracking.\r\n");
 	fprintf(stdout, "F - Xterm 1006 extension.\r\n");
-	fprintf(stdout, "G - rxvt-unicode extension.\r\n");
 	fprintf(stdout, "I - Focus tracking.\r\n");
 	fprintf(stdout, "Q - Quit.\r\n");
 	fprintf(stdout, "%s", _VTE_CAP_ESC "8");
@@ -208,7 +201,7 @@ parse_esc(guint8 *data,
         if (data[2] == 'M')
                 return parse_legacy_mouse_mode(data, len);
 
-        /* FIXME: add support for xterm extended mode (1006) and urxvt mode (1015) */
+        /* FIXME: add support for xterm extended mode (1006) */
 
         if (data[2] == 'I' || data[2] == 'O') {
                 fprintf(stdout, "focus %s\r\n", data[2] == 'I' ? "in" : "out");
@@ -313,12 +306,6 @@ parse(void)
 					0 : tracking_xterm_ext;
 			i++;
 			break;
-		case 'G':
-		case 'g':
-			tracking_mode = (tracking_mode == tracking_urxvt) ?
-					0 : tracking_urxvt;
-			i++;
-			break;
 		case 'I':
 		case 'i':
 			tracking_focus_mode = !tracking_focus_mode;
@@ -336,8 +323,8 @@ parse(void)
                                 i += consumed;
                                 break;
                         }
-                        /* else fall-trough */
                 }
+                        /* fallthrough */
 		default:
                         i += print_data(&bytes->data[i], bytes->len - i);
 			fprintf(stdout, "\r\n");
@@ -352,6 +339,7 @@ parse(void)
 
 static struct termios tcattr, original;
 
+G_GNUC_NORETURN
 static void
 sigint_handler(int signum)
 {
@@ -376,17 +364,7 @@ main(int argc, char **argv)
 
 	original = tcattr;
 	signal(SIGINT, sigint_handler);
-	/* Here we approximate what cfmakeraw() would do, for the benefit
-	 * of systems which don't actually provide the function. */
-	tcattr.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP |
-			    INLCR | IGNCR | ICRNL | IXON);
-	tcattr.c_oflag &= ~(OPOST);
-	tcattr.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
-	tcattr.c_cflag &= ~(CSIZE | PARENB);
-	tcattr.c_cflag |= CS8;
-#ifdef HAVE_CFMAKERAW
 	cfmakeraw(&tcattr);
-#endif
 	if (tcsetattr(STDIN_FILENO, TCSANOW, &tcattr) != 0) {
 		perror("tcsetattr");
 		return 1;
