@@ -34,6 +34,7 @@
 #include "vteptyinternal.hh"
 #include "vtetypes.hh"
 #include "vtespawn.hh"
+#include "reaper.hh"
 
 #include <assert.h>
 #include <sys/types.h>
@@ -127,16 +128,6 @@ Pty::child_setup() const noexcept
         if (masterfd == -1)
                 _exit(127);
 
-        if (grantpt(masterfd) != 0) {
-                _vte_debug_print(VTE_DEBUG_PTY, "%s failed: %m\n", "grantpt");
-                _exit(127);
-        }
-
-	if (unlockpt(masterfd) != 0) {
-                _vte_debug_print(VTE_DEBUG_PTY, "%s failed: %m\n", "unlockpt");
-                _exit(127);
-        }
-
         if (!(m_flags & VTE_PTY_NO_SESSION)) {
                 /* This starts a new session; we become its process-group leader,
                  * and lose our controlling TTY.
@@ -488,6 +479,8 @@ Pty::spawn(char const* directory,
                                                      cancellable,
                                                      err)) {
                 if (spawn_flags & VTE_SPAWN_REQUIRE_SYSTEMD_SCOPE) {
+			vte_reaper_add_child(pid);
+
                         auto pgrp = getpgid(pid);
                         if (pgrp != -1) {
                                 kill(-pgrp, SIGHUP);
@@ -619,6 +612,18 @@ fd_set_cpkt(int fd)
 static int
 fd_setup(int fd)
 {
+        if (grantpt(fd) != 0) {
+                vte::util::restore_errno errsv;
+                _vte_debug_print(VTE_DEBUG_PTY, "%s failed: %m\n", "grantpt");
+                return -1;
+        }
+
+        if (unlockpt(fd) != 0) {
+                vte::util::restore_errno errsv;
+                _vte_debug_print(VTE_DEBUG_PTY, "%s failed: %m\n", "unlockpt");
+                return -1;
+        }
+
         if (fd_set_cloexec(fd) < 0) {
                 vte::util::restore_errno errsv;
                 _vte_debug_print(VTE_DEBUG_PTY,
@@ -703,6 +708,18 @@ _vte_pty_open_posix(void)
                 return -1;
         }
 
+        if (grantpt(fd) != 0) {
+                vte::util::restore_errno errsv;
+                _vte_debug_print(VTE_DEBUG_PTY, "%s failed: %m\n", "grantpt");
+                return {};
+        }
+
+        if (unlockpt(fd) != 0) {
+                vte::util::restore_errno errsv;
+                _vte_debug_print(VTE_DEBUG_PTY, "%s failed: %m\n", "unlockpt");
+                return {};
+        }
+
 	_vte_debug_print(VTE_DEBUG_PTY, "Allocated pty on fd %d.\n", (int)fd);
 
         return fd.steal();
