@@ -46,6 +46,12 @@ void vte_terminal_match_set_cursor(VteTerminal *terminal,
                                    int tag,
                                    GdkCursor *cursor) _VTE_GNUC_NONNULL(1);
 
+_VTE_DEPRECATED
+_VTE_PUBLIC
+void vte_terminal_match_set_cursor_type(VteTerminal *terminal,
+					int tag,
+                                        GdkCursorType cursor_type) _VTE_GNUC_NONNULL(1);
+
 _VTE_DEPRECATED
 _VTE_PUBLIC
 char *vte_terminal_match_check(VteTerminal *terminal,
@@ -105,6 +111,57 @@ _VTE_PUBLIC
 void vte_terminal_set_geometry_hints_for_window(VteTerminal *terminal,
                                                 GtkWindow *window) _VTE_GNUC_NONNULL(1) _VTE_GNUC_NONNULL(2);
 
+_VTE_DEPRECATED
+_VTE_PUBLIC
+const char *vte_terminal_get_icon_title(VteTerminal *terminal) _VTE_GNUC_NONNULL(1);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+gboolean vte_terminal_set_encoding(VteTerminal *terminal,
+                                   const char *codeset,
+                                   GError **error) _VTE_GNUC_NONNULL(1);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+const char *vte_terminal_get_encoding(VteTerminal *terminal) _VTE_GNUC_NONNULL(1);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+char *vte_terminal_get_text_include_trailing_spaces(VteTerminal *terminal,
+						    VteSelectionFunc is_selected,
+						    gpointer user_data,
+						    GArray *attributes) _VTE_GNUC_NONNULL(1) G_GNUC_MALLOC;
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+void vte_terminal_set_rewrap_on_resize(VteTerminal *terminal,
+                                       gboolean rewrap) _VTE_GNUC_NONNULL(1);
+_VTE_DEPRECATED
+_VTE_PUBLIC
+gboolean vte_terminal_get_rewrap_on_resize(VteTerminal *terminal) _VTE_GNUC_NONNULL(1);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+void vte_terminal_set_allow_bold(VteTerminal *terminal,
+                                 gboolean allow_bold) _VTE_GNUC_NONNULL(1);
+_VTE_DEPRECATED
+_VTE_PUBLIC
+gboolean vte_terminal_get_allow_bold(VteTerminal *terminal) _VTE_GNUC_NONNULL(1);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+void vte_terminal_feed_child_binary(VteTerminal *terminal,
+                                    const guint8 *data,
+                                    gsize length) _VTE_GNUC_NONNULL(1);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+char **vte_get_encodings(gboolean include_aliases);
+
+_VTE_DEPRECATED
+_VTE_PUBLIC
+gboolean vte_get_encoding_supported(const char *encoding);
+
 G_END_DECLS
 
 #undef _VTE_DEPRECATED
